@@ -19,12 +19,12 @@ local lua_util = require "lua_util"
 local logger = require "rspamd_logger"
 local lua_redis = require "lua_redis"
 local dmarc_common = require "plugins/dmarc"
-local lupa = require "lupa"
 local rspamd_mempool = require "rspamd_mempool"
 local rspamd_url = require "rspamd_url"
 local rspamd_text = require "rspamd_text"
 local rspamd_util = require "rspamd_util"
 local rspamd_dns = require "rspamd_dns"
+local fun = require "fun"
 
 local N = 'dmarc_report'
 
@@ -176,8 +176,6 @@ local function escape_xml(input)
 
   return ''
 end
--- Enable xml escaping in lupa templates
-lupa.filters.escape_xml = escape_xml
 
 -- Creates report XML header
 local function report_header(reporting_domain, report_start, report_end, domain_policy)
@@ -211,7 +209,10 @@ local function report_header(reporting_domain, report_start, report_end, domain_
     report_end = report_end,
     domain_policy = domain_policy,
     reporting_domain = reporting_domain,
-  }, true)
+  }, true, false,
+      {
+        escape_xml = escape_xml
+      })
 end
 
 -- Generate xml entry for a preprocessed redis row
@@ -248,7 +249,10 @@ local function entry_to_xml(data)
   </auth_results>
 </record>
 ]]
-  return lua_util.jinja_template(xml_template, { data = data }, true)
+  return lua_util.jinja_template(xml_template, { data = data }, true,
+      false, {
+        escape_xml = escape_xml
+      })
 end
 
 -- Process a report entry stored in Redis splitting it to a lua table
@@ -285,16 +289,15 @@ end
 
 -- Process a single rua entry, validating in DNS if needed
 local function process_rua(dmarc_domain, rua)
-  local parts = lua_util.str_split(rua, ',')
-
   -- Remove size limitation, as we don't care about them
   local addrs = {}
-  for _, rua_part in ipairs(parts) do
+  for _, rua_part in fun.map(lua_util.str_trim, lua_util.str_split(rua, ',')) do
     local u = rspamd_url.create(pool, rua_part:gsub('!%d+[kmg]?$', ''))
+    local u2 = rspamd_url.create(pool, dmarc_domain)
     if u and (u:get_protocol() or '') == 'mailto' and u:get_user() then
       -- Check each address for sanity
-      if dmarc_domain == u:get_tld() or dmarc_domain == u:get_host() then
-        -- Same domain - always include
+      if u:get_tld() == u2:get_tld() then
+        -- Same eSLD - always include
         table.insert(addrs, u)
       else
         -- We need to check authority
@@ -327,7 +330,7 @@ local function process_rua(dmarc_domain, rua)
         end
       end
     else
-      logger.errx('invalid rua url: "%s""', tostring(u or 'null'))
+      logger.errx('invalid rua url: "%s""', rua_part)
     end
   end
 
@@ -341,18 +344,15 @@ end
 -- Validate reporting domain, extracting rua and checking 3rd party report domains
 -- This function returns a full dmarc record processed + rua as a list of url objects
 local function validate_reporting_domain(reporting_domain)
-  -- Now check the domain policy
-  -- DMARC domain is a esld for the reporting domain
-  local dmarc_domain = rspamd_util.get_tld(reporting_domain)
   local is_ok, results = rspamd_dns.request({
     config = rspamd_config,
     session = rspamadm_session,
     type = 'txt',
-    name = '_dmarc.' .. dmarc_domain,
+    name = '_dmarc.' .. reporting_domain,
   })
 
   if not is_ok or not results then
-    logger.errx('cannot resolve _dmarc.%s: %s', dmarc_domain, results)
+    logger.errx('cannot resolve _dmarc.%s: %s', reporting_domain, results)
     return nil
   end
 
@@ -360,7 +360,7 @@ local function validate_reporting_domain(reporting_domain)
     local processed, rec = dmarc_common.dmarc_check_record(rspamd_config, r, false)
     if processed and rec.rua then
       -- We need to check or alter rua if needed
-      local processed_rua = process_rua(dmarc_domain, rec.rua)
+      local processed_rua = process_rua(reporting_domain, rec.rua)
       if processed_rua then
         rec = rec.raw_elts
         rec.rua = processed_rua
@@ -536,10 +536,15 @@ local function prepare_report(opts, start_time, end_time, rep_key)
     message_id = rspamd_util.random_hex(16) .. '@' .. report_settings.msgid_from,
     report_start = start_time,
     report_end = end_time
-  }, true)
+  }, true,
+      false, {
+        escape_xml = escape_xml
+      })
   local rfooter = lua_util.jinja_template(report_footer, {
     uuid = uuid,
-  }, true)
+  }, true, false, {
+    escape_xml = escape_xml
+  })
   local message = rspamd_text.fromtable {
     (rhead:gsub("\n", "\r\n")),
     rspamd_util.encode_base64(rspamd_util.gzip_compress(xml_to_compress), 73),
@@ -736,4 +741,4 @@ return {
   aliases = { 'dmarc_reporting' },
   handler = handler,
   description = parser._description
-}
\ No newline at end of file
+}
